EXPORTS
srtp_init
srtp_protect
srtp_unprotect
srtp_create
srtp_add_stream
srtp_remove_stream
crypto_policy_set_rtp_default
crypto_policy_set_rtcp_default
crypto_policy_set_aes_cm_128_hmac_sha1_32
crypto_policy_set_aes_cm_128_null_auth
crypto_policy_set_aes_cm_256_hmac_sha1_32
crypto_policy_set_aes_cm_256_hmac_sha1_80
crypto_policy_set_null_cipher_hmac_sha1_80
crypto_policy_set_rtp_default
srtp_dealloc
srtp_get_stream
srtp_protect_rtcp
srtp_unprotect_rtcp
srtp_install_event_handler
crypto_kernel_init
crypto_kernel_shutdown
crypto_kernel_status
crypto_kernel_list_debug_modules
crypto_kernel_load_cipher_type
crypto_kernel_load_auth_type
crypto_kernel_load_debug_module
crypto_kernel_alloc_cipher
crypto_kernel_alloc_auth
crypto_kernel_set_debug_module
crypto_get_random
rand_source_init
rand_source_get_octet_string
rand_source_deinit
x917_prng_init
x917_prng_get_octet_string
ctr_prng_init
ctr_prng_get_octet_string
cipher_output
cipher_get_key_length
cipher_type_self_test
cipher_bits_per_second
auth_get_key_length
auth_get_tag_length
auth_get_prefix_length
auth_type_self_test
auth_type_get_ref_count
stat_test_monobit
stat_test_poker
stat_test_runs
stat_test_rand_source
stat_test_rand_source_with_repetition
err_reporting_init
err_report
key_limit_set
key_limit_clone
key_limit_check
key_limit_update
rdbx_init
rdbx_estimate_index
rdbx_check
rdbx_add_index
index_init
index_advance
index_guess
octet_get_weight
octet_string_hex_string
v128_bit_string
v128_hex_string
nibble_to_hex_char
hex_string_to_octet_string
v128_copy_octet_string
v128_left_shift
v128_right_shift
octet_string_is_eq
octet_string_set_to_zero
rdb_init
rdb_check
rdb_add_index
rdb_increment
rdb_get_value
aes_expand_encryption_key
aes_expand_decryption_key
srtp_aes_encrypt
srtp_aes_decrypt
aes_icm_context_init
aes_icm_set_iv
aes_icm_encrypt
aes_icm_output
aes_icm_dealloc
aes_icm_encrypt_ismacryp
aes_icm_alloc_ismacryp
crypto_alloc
crypto_free
